/*
  0.17: cleared code from 30000bytes to 25320 bytes, 16 Tracks now
  0.18: changed internal clock to interrupt routine
  0.19: switched setup to getDataValue() and fixed edit of multiple tracks
  0.20: redid the write and play stuff
  0.21: redid EEPROM organization
  0.22: fixed transpose finished play mode; included transpose and velocity tracks
  0.23: switched Potentiometer handling to new concept
  0.25: global transpose can use transpose track
  0.26: Trackpreparation after offTrigger of former step to improve the timing.
  0.27: MIDI Clock-Send and DIN-Sync/VMC-Clock, Global DividerDivider,  MIDI ClockReceive
  0.28: Some minor bugfixing.
  0.29: Menu scrolling by STOP+CV2, fixed track-copy/paste
  0.30: Enabled Write a sequencer while PLAY. ToDo: establish pause/continue on next bar instead of sequence end. Exiting WRITE mode with PLAY while runmode=1 must set the selected Track in wait for start modus
  0.31: Few minor bugfixes. Value edit now always works with STOP+CV2.2
  0.32: invalue_2 changed
  0.33: returned to single file
  0.34: fixed velocity source global
  ToDo
   Add separate Save/Load all track configs
   Add separate Save/Load all tracks stepvalues.
   Add separate Save/Load single tracks.
   Maybe:
   two level menu structure
   or
   CV1 scrolls thru menues
*/


#include <TimerOne.h>
#include <EEPROM.h>
#include <MIDI.h>
#include <LiquidCrystal.h>
MIDI_CREATE_DEFAULT_INSTANCE();
LiquidCrystal lcd(2, 3, 4, 5, 6, 7);

// Define inputs and outputs
#define button1Pin 8 //PLAY
#define button2Pin 9 //STOP
#define button3Pin 12 //WRITE
#define button4Pin 11 //F1
#define button5Pin 10 //F2
#define clockInPin 13 //CLOCK-IN
#define TempoCVPin A0 // =Tempo potentiometer
#define CV1Pin A1 // (5V or CV1-In) via CV1 potentiometer
#define CV2Pin A2 // (5V or CV2-In) via CV2 potentiometer


#define dinClockPin  A3 //VMC-BUS
#define dinGatePin  A4 //VMC-BUS
#define clockOutPin A5 //->sent via the clock-socket-switch to clockInPin

#define input0 A0 // =Tempo potentiometer
#define input1 A1 // (5V or CV1-In) via CV1 potentiometer
#define input2 A2 // (5V or CV2-In) via CV2 potentiometer
uint16_t input[3];



// Variables to handle the potentiometers
//uint16_t invalueRaw[3]; /* Used in function poti adjust */
uint16_t invalueOffset[3]; /* The difference between 1023 and the real AD-ed value */
uint16_t adoptedRead[3]; /* The calculated value that makes 1023 from the real AD-ed value */
uint8_t invalue[3]; /* The value 0-127 calculated from adoptedRead[8] using invalueOffset[8] */
uint8_t invaluePrev[3];
unsigned long lastRead[3]; /* used for correcting false retriggering*/
unsigned long thisRead[3]; /* used for correcting false retriggering*/
unsigned long CV[3];

/*
  //Offset variables for the CV ins, for adopting to potentiometer variations
  unsigned long CV1raw;
  unsigned long CV2raw;
  unsigned long TempoCVraw;

  //Offset variables for the CV ins, for adopting to potentiometer variations
  unsigned long maxOffTempoCV;
  unsigned long maxOffCV1;
  unsigned long maxOffCV2;

  // Variables to handle the potentiometers
  unsigned long CV1;
  unsigned long CV2;
  unsigned long TempoCV;
*/


// Variables to handle the buttons and digital inputs
byte button1State = 0;
byte button2State = 0;
byte button3State = 0;
byte button4State = 0;
byte button5State = 0;
byte clockInState = 0;
byte button1StatePrev = 0;
byte button2StatePrev = 0;
byte button3StatePrev = 0;
byte button4StatePrev = 0;
byte button5StatePrev = 0;
byte clockInStatePrev = 0;

// Variable required for the clock generator
uint8_t tempo;
long period;
uint16_t midiclockcounter = 0; // Counts incoming MIDI-Clock or DIN-Sync clock related to the divider settings
uint16_t intclockcounter = 0; // Counts the internal clock related to the divider settings
uint16_t dinclockcounter = 0; // Counts the internal clock for 24ppq for DIN-Sync out and MIDI-clock out
uint16_t barClockCount = 0; // counts for one bar (either MIDI, internal or DIN-Sync) for play/continue
uint16_t barCount = 96; // the maxValue for the barClockCount; 96 is one 4/4 Bar

unsigned long clockfactor = 1;
byte startFlag = 0;




// Variables for systemwider setup parameters

byte inchannel = 1;
//byte notelength = 1;
byte transposemodeglobal = 3; //Buttonenabled
byte transposecenter = 60;
byte noteoffmode = 0; //legato
byte velocitymodeglobal = 1; //Random
byte notethrumode = 3; //Rec+Play
byte controlthrumode = 1;
byte programthrumode = 0;
byte bendthrumode = 1;
byte otherthrumode = 0;
byte clockmode = 0;


/* Variable used for velocity control */
byte center;
byte range;

// Variables for track data
const byte maxSteps = 32;
const byte maxTracks = 16;
//byte stepvalue[maxSteps][maxTracks];
byte stepvalue[maxSteps][maxTracks];
byte noteOff[maxTracks];
byte ccnumber[maxTracks];
//byte ccnumberall = 7;
byte clockdivider[maxTracks];
byte inclockdivider = 0;
byte clockcounter[maxTracks];
byte laststep[maxTracks];
byte outchannel[maxTracks];
//byte outchannelall = 10;
uint8_t stepnumber[maxTracks];
byte tracktype[maxTracks];
byte tracktypeall = 0;
int transpose[maxTracks];
int transposeall = 0;
byte transposesource[maxTracks];
byte velocity[maxTracks];
byte velocityall = 64;
byte velocitysource[maxTracks];
byte currentvalue[maxTracks];
byte osmoseenable = 0;
byte sendclock = 0;
byte startstopmode = 0;


// Variables for sequence handling
byte playedtrack; // used like a normal for i in ... counter, but the name helps understanding the code
byte selectedtrack;
byte pause[maxTracks]; // 0=normal play, 1=steps are counted, but no midi is sent, 2=will play again after next cycle
//byte waitEndOfPause[maxTracks]; // should be replaced by pause[track]==2
byte playonstart[maxTracks];
byte writeStepNumber = 0;
byte stepBack = 0;
byte stepLCDposition;
byte ontrigger[maxTracks];
byte offtrigger[maxTracks];
byte i = 0;
byte j = 0;

// Variables to handle the generic status of the software
byte setupmode = 0;
byte comingfromsetupmode = 0; //  this is used to supress play or stop when returning from setup with PLAY+STOP
byte menu = 0;
byte menuPrev = 0;
byte edit = 0;
byte runmode = 0; // 0=Stop, 1=Play
byte writemode = 0; // 1=Record Trigger, 2=Record Velocity
byte writeselect = 0;
byte adjuststep = 0; //used for the poti adjustment procedure
byte myvalue;
byte maxValue;
byte minValue;
byte incten;
byte runover;
byte saveMemory;
byte saveValue;
byte saveValueMulti[maxTracks];
byte saveMulti; // when this is 1 at leaving the edit mode => this menu requires saving the parameters of all tracks
byte displaymode = 0;




void setup() {

  invalueOffset[0] = EEPROM.read(1);
  invalueOffset[1] = EEPROM.read(2);
  invalueOffset[2] = EEPROM.read(3);

  lcd.begin(16, 2);
  MIDI.begin();
  MIDI.setInputChannel(inchannel);
  if (otherthrumode == 0) {
    MIDI.setThruFilterMode(0);
    MIDI.turnThruOff();
  }
  if (otherthrumode == 1) {
    MIDI.setThruFilterMode(3);
    MIDI.turnThruOn(3);
  }

  pinMode(button1Pin, INPUT);
  pinMode(button2Pin, INPUT);
  pinMode(button3Pin, INPUT);
  pinMode(button4Pin, INPUT);
  pinMode(button5Pin, INPUT);
  pinMode(clockInPin, INPUT);
  pinMode(input0, INPUT);
  pinMode(input1, INPUT);
  pinMode(input2, INPUT);
  pinMode(clockOutPin, OUTPUT);

  input[0] = input0;
  input[1] = input1;
  input[2] = input2;
  tempo = 120;

  /* TIMER FOR THE INTERNAL CLOCK */
  Timer1.initialize(100000);
  Timer1.attachInterrupt( timerIsr );
  period = (312500 / tempo);
  Timer1.setPeriod(period);
  transposecenter = EEPROM.read(7);
  triggerreset();
  loadsequence();
  loadsettings();
  clockmode = 0;
  setClockFactor();
  /* PRINT FIRMWARE VERSION */
  lcd.setCursor(0, 0);
  lcd.print(F("SQ-4m V0.33     "));
  lcd.setCursor(0, 1);
  //lcd.print(inclockdivider); //DEBUG-DISPLAY AFTER START
  lcd.print(F("                "));
  delay(1000);
  clearscreen();
}

/* ------ FULL SYSTEM RESET ------- */
void(* resetFunc)  (void) = 0;



void loop() {
  /* DEBUGGING OUTPUTS - COMMENTED NORMALLY */
  /* 
  lcd.setCursor(11, 0);
  //lcd.print(barClockCount / 48);
  lcd.print(comingfromsetupmode);
  //lcd.print(button5StatePrev);
  //lcd.print(F(" "));
  //*/



  /* READ BUTTONS */
  button1State = !digitalRead(button1Pin);
  button2State = !digitalRead(button2Pin);
  button3State = !digitalRead(button3Pin);
  button4State = !digitalRead(button4Pin);
  button5State = !digitalRead(button5Pin);

  /* READ THE POTENTIOMETERS */
  for (i = 0 ; i <= 2 ; i++) {
    lastRead[i] = thisRead[i];
    thisRead[i] = analogRead(input[i]);
    invaluePrev[i] = invalue[i];
    adoptedRead[i] = int(thisRead[i] + ((thisRead[i] + 1.0) / (1023.0 - (invalueOffset[i] + 1) )) * invalueOffset[i]);
    if (adoptedRead[i] > 1023) adoptedRead[i] = 1023;
    if (i == 0) {
      invalue[i] = int(adoptedRead[i] / 4);
      if (invalue[i] > 255) invalue[i] = 255;
      if (invalue[i] < 1) invalue[i] = 1;
    }
    if (i > 0) {
      invalue[i] = int(adoptedRead[i] / 8);
      if (invalue[i] > 127) invalue[i] = 127;
    }
    if (invalue[i] < 0) invalue[i] = 0;
  }

  if (button2State == 0) center = (invalue[1]);
  if (button2State == 0) range = (invalue[2]);

  /* ------------ FULL SYSTEM RESET WHEN ALL BUTTONS ARE PRESSED ------------ */
  if (button1State == 1 && button2State == 1 && button3State == 1 && button4State == 1 && button5State == 1) {
    resetFunc();
  }

  if ( invalue[0] != invaluePrev[0]) { // A change at the Tempo potentiometer sets a new tempo
    //period = (312500 /invalue[0]);
    if (invalue[0] > 0 && invalue[0] < 256) tempo = invalue[0];
    Timer1.setPeriod(312500 / tempo);
    if (displaymode == 0 && transpose == 0) {
      lcd.setCursor(5, 0);
      lcd.print(tempo);
      lcd.print(F(" "));
    }
  }




  /* BUTTONS IN THE NORMAL MENUS */


  if (setupmode == 0) { /* Includes STOP MODE and WRITE MODE */
    /* PRESS WRITE WHILE HOLDING F1 DELETES THE PATTERN */
    if (button1State == 0 && button2State == 0 && button3State == 1 && button3StatePrev == 0 && button4State == 1 && button5State == 0) {
      button3StatePrev = 1;
    }
    if (button1State == 0 && button2State == 0 && button3State == 0 && button3StatePrev == 1 && button4State == 1 && button5State == 0) {
      button3StatePrev = 0;
      deletesequence();
    }

    /* HOLD  STOP AND PRESS WRITE SENDS MIDI-RESET */
    if (button3State == 0 && button3StatePrev == 1 && button2State == 1 && button1State == 0 && button4State == 0 && button5State == 0) midireset();

    /* TIE plus REST will switch to setup in Playmode and Stopmode
      if (writemode == 0) {
      if (button4State == 1 && button4StatePrev == 0 && button5State == 1) {
        button4StatePrev = 1;
        setupmode = 1;
        clearscreen();
      }
      if (button4State == 0 && button4StatePrev == 1) {
        button4StatePrev = 0;
      }
      }
      //*/



    /* PRINT PLAY OR STOP TO DISPLAY */
    lcd.setCursor(0, 0);
    if (runmode == 0 && writemode == 0) {
      lcd.setCursor(0, 0);
      lcd.print(F("STOP"));
    }
    if (runmode == 1  && pause[selectedtrack] == 0 ) {
      lcd.setCursor(0, 0);
      lcd.print(F("PLAY"));
    }
    if (runmode == 1  && pause[selectedtrack] == 1) {
      lcd.setCursor(0, 0);
      lcd.print(F("play"));
    }
    if (runmode == 1  && pause[selectedtrack] == 2) {
      lcd.setCursor(0, 0);
      lcd.print(F("plAY"));
    }



    if (writemode == 0) {
      /* TIE plus REST will switch to setup in Playmode and Stopmode */
      if (button4State == 1 && button5State == 1 && button1State == 0 && button2State == 0 && button3State == 0) {
        //normaltrack = selectedtrack;
        //selectedtrack = setuptrack;
        setupmode = 1;
        clearscreen();
      }

      /* F2 PRESSED + BUTTON START/STOP SELECTS TRACK */
      if (button5State == 1 && button1State == 1 && button1StatePrev == 0 && writemode == 0 && button2State == 0 && button3State == 0 && button4State == 0) {
        if (selectedtrack == 0) selectedtrack = maxTracks;
        selectedtrack--;
        button1StatePrev = 1;
      }
      if (button1State == 0 && button1StatePrev == 1) button1StatePrev = 0;

      if (button5State == 1 && button2State == 1 && button2StatePrev == 0 && writemode == 0 && button1State == 0 && button3State == 0 && button4State == 0) {
        selectedtrack++;
        if (selectedtrack == maxTracks) selectedtrack = 0;
        button2StatePrev = 1;
      }
      if (button2State == 0 && button2StatePrev == 1) button2StatePrev = 0;

      lcd.setCursor(12, 0);

      lcd.print(F("Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
    }

    /* F1 PRESSED + STOP MUTES ALL TRACKS*/
    if (button4State == 1 && button2State == 1 && button2StatePrev == 0 && writemode == 0 && button1State == 0 && button3State == 0 && button5State == 0) {
      for (i = 0; i <= (maxTracks - 1); i++) {
        pause[i] = 1;
      }
      button2StatePrev = 1;
      lcd.setCursor(0, 1);
      lcd.print(F("all tracks muted"));
    }
    if (button2State == 0 && button2StatePrev == 1) button2StatePrev = 0;


    /* F1 PRESSED + START UNMUTES ALL TRACKS AND STARTS PLAYING*/
    if (button4State == 1 && button1State == 1 && button1StatePrev == 0 && writemode == 0 && button2State == 0 && button3State == 0 && button5State == 0) {
      for (i = 0; i <= (maxTracks - 1); i++) {
        pause[i] = 0;
      }
      lcd.setCursor(0, 1);
      lcd.print(F("all tracks play "));
      button1StatePrev = 1;
    }
    if (button2State == 0 && button2StatePrev == 1) button2StatePrev = 0;


    if ( comingfromsetupmode > 0 && button1State == 0 && button2State == 0) {
      comingfromsetupmode = 0;
      clearscreen();
    }



    /* PLAY BUTTON IN STOP MODE*/

    if (button1State == 1 && button1StatePrev == 0 && runmode == 0 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0 && comingfromsetupmode == 0) { //PLAY
      button1StatePrev = 1;
      if (sendclock >= 1) startFlag = 1; // send MIDI-Start command
      runmode = 1;
      for (i = 0; i <= (maxTracks - 1); i++) {
        //if (noteoffmode == 0) stepnumber[i] = -1;
        //if (noteoffmode > 0) stepnumber[i] = 0
        stepnumber[i] = 0;
        if (playonstart[i] == 0) {
          pause[i] = 0;
        }
        clockcounter[i] = 0;
      }
      barClockCount = 0;
      clearscreen();

    }
    if (button1State == 0 && button1StatePrev == 1)  button1StatePrev = 0;

    /* PLAY BUTTON IN WRITE MODE */
    if (button1State == 1 && button1StatePrev == 0 && writemode == 1 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0) { //PLAY
      button1StatePrev = 1;
      writemode = 0;
      for ( i = 0; i <= maxTracks; i++) {
        if (noteoffmode == 0) stepnumber[i] = -1;
        if (noteoffmode > 0) stepnumber[i] = 0;
        clockcounter[i] = 0;
      }
      runmode = 1;
      clearscreen();
      //if (button2State == 0) stepnumber[selectedtrack] = 0; // DO NOT RESET THE SEQUENCE IF LEAVING THE SETUP
    }
    if (button1State == 0 && button1StatePrev == 1)  button1StatePrev = 0;



    /* WRITE BUTTON IN STOP MODE */
    // if (button3State == 1 && button3StatePrev == 0 && button1State == 0 && button2State == 0 && button4State == 0 && button5State == 0 && runmode == 0  && writemode == 0 && setupmode == 0) { //WRITE
    /* WRITE BUTTON IN STOP OR PLAY MODE */
    if (button3State == 1 && button3StatePrev == 0 && writemode == 0 && setupmode == 0 && button1State == 0 && button2State == 0 && button4State == 0 && button5State == 0) { //WRITE
      button3StatePrev = 1;
      stepnumber[selectedtrack] = 0;
      writemode = 1;
      clearscreen();
    }
    if (button3State == 0 && button3StatePrev == 1)  button3StatePrev = 0;

    /* STOP BUTTON IN STOP OR PLAY MODE*/
    if (button2State == 1 && button2StatePrev == 0 && button1State == 0 && button3State == 0 && button4State == 0 && button5State == 0 && comingfromsetupmode == 0) { //STOP
      button2StatePrev = 1;
      runmode = 0;
      clockInStatePrev = 0;
      triggerreset();
      for ( playedtrack = 0 ; playedtrack <= maxTracks - 1 ; playedtrack++ ) {
        if (tracktype[playedtrack] == 0 && noteOff[playedtrack] < 128 && noteOff[playedtrack] > 0) {
          MIDI.sendNoteOff(noteOff[playedtrack], velocity[playedtrack], outchannel[playedtrack]);
          noteOff[playedtrack] = 128;
        }
      }
      for (i = 0; i <= (maxTracks - 1); i++) {
        stepnumber[i] = 0;
        if (playonstart == 0) {
          pause[i] = 0;
        }
      }
    }
    if (button2State == 0 && button2StatePrev == 1) button2StatePrev = 0;




    /* ------------------------ STOP MODE STARTS HERE ------------------------ */
    if (runmode == 0 && writemode == 0) {
      lcd.setCursor(0, 0);
      lcd.print(F("STOP "));

      /* MIDITHRU IN STOP MODE*/
      if (MIDI.read()) {
        byte type = MIDI.getType();
        switch (type) {
          case midi::NoteOn:
            if (notethrumode > 0) MIDI.sendNoteOn(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
            break;

          case midi::NoteOff:
            if (notethrumode > 0) MIDI.sendNoteOff(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
            break;

          case midi::ControlChange:
            if ( controlthrumode == 1) MIDI.sendControlChange(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
            break;

          case midi::ProgramChange:
            if ( programthrumode == 1) MIDI.sendProgramChange(MIDI.getData1(), outchannel[selectedtrack]);
            break;

          case midi::PitchBend:
            if ( bendthrumode == 1) MIDI.sendPitchBend( (((MIDI.getData2() * 128) + MIDI.getData1() ) + 8192), outchannel[selectedtrack]);
            break;

          case midi::Start:
            if (clockmode > 0 ) {
              midiclockcounter = 0;
              lcd.setCursor(0, 0);
              /*
                for (i = 0; i <= (maxTracks - 1); i++) {
                if (noteoffmode == 0) stepnumber[i] = -1;
                if (noteoffmode > 0) stepnumber[i] = 0;
                byte tempvelocity = (center - ( random(1, 126) *  ((range + 1) / 128.0) ) / 2) + ((random(1, 126) *  ((range + 1) / 128) ));
                if (tempvelocity < 1) tempvelocity = 1;
                if (tempvelocity > 127) tempvelocity = 127;
                velocity[i] = tempvelocity;
                if (playonstart[i] == 0) {
                  pause[i] = 0;
                  waitEndOfPause[i] = 0;
                }
                }
                //*/
              runmode = 1;
              break;
            }
        }    /* END OF MIDITHRU IN STOP MODE */


      }
    }/* END OF STOP MODE */
    /* ------------------------ STOP MODE ENDS HERE --------------------------- */

    /* ------------------------ WRITE MODE STARTS HERE ------------------------ */
    // if (runmode == 0 && writemode == 1) {
    if (writemode == 1) {
      lcd.setCursor(0, 0);
      lcd.print(F("WRITE"));
      lcd.print(F(" "));
      /*
        if (tracktype[selectedtrack] > 1) {
        lcd.print(F("+"));
        lcd.print(invalue[1]);
        lcd.print(F(" "));
        }
      */
      stepLCDposition = stepnumber[selectedtrack] % 16;

      /* WRITE NoteOn and NoteOff */
      if (MIDI.read()) {
        byte type = MIDI.getType()  ;
        switch (type) {
          case midi::NoteOn:
            if (MIDI.getChannel() == inchannel) {
              if (tracktype[selectedtrack] == 0 || tracktype[selectedtrack] == 3) {
                // MIDI THRU WHILE RECORD
                if (notethrumode > 1) MIDI.sendNoteOn(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
                // IF THIS TRACK IS A NOTE TRACK THEN WRITE A MIDI NOTE
                stepvalue[stepnumber[selectedtrack]][selectedtrack] = MIDI.getData1();

                lcd.setCursor(6, 0);
                lcd.print(F("#"));
                lcd.print(stepvalue[stepnumber[selectedtrack]][selectedtrack]);
                lcd.print(F(" "));
                lcd.setCursor(13, 0);
                lcd.print(stepnumber[selectedtrack] + 1);
                lcd.print(F(" "));
                printstep(stepLCDposition, 43); // Print a "+"
                stepnumber[selectedtrack]++;
                laststep[selectedtrack] = stepnumber[selectedtrack];
              }
            }
            break;
          case midi::NoteOff:
            if (notethrumode > 0) MIDI.sendNoteOff(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
            break;

          case midi::ControlChange:
            if ( controlthrumode == 1) MIDI.sendControlChange(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
            break;

          case midi::ProgramChange:
            if ( programthrumode == 1) MIDI.sendProgramChange(MIDI.getData1(), outchannel[selectedtrack]);
            break;

          case midi::PitchBend:
            if ( bendthrumode == 1) MIDI.sendPitchBend( (((MIDI.getData2() * 128) + MIDI.getData1() ) + 8192), outchannel[selectedtrack]);
            break;

        }
      }


      // IF THIS TRACK IS A CC TRACK OR VELOCITY TRACK THEN SELECT THE CV1 VALUE THAT WE WANT TO WRITE
      if (tracktype[selectedtrack] == 1 || tracktype[selectedtrack] == 2) {
        if (button4State == 1 && button1State == 0 && button2State == 0 && button3State == 0 && button5State == 0) {
          button4StatePrev = 1;
          lcd.setCursor(6, 0);
          lcd.print(F("#"));
          lcd.print(invalue[2]);
          lcd.print(F(" "));
        }
        if (button4State == 0 && button4StatePrev == 1 && button1State == 0 && button2State == 0 && button3State == 0 && button5State == 0) {
          button4StatePrev = 0;
          if (tracktype[selectedtrack] == 1 || tracktype[selectedtrack] == 2) {
            stepvalue[stepnumber[selectedtrack]][selectedtrack] = (invalue[2]);
          }
          lcd.setCursor(13, 0);
          lcd.print(stepnumber[selectedtrack] + 1);
          lcd.print(F(" "));
          printstep(stepLCDposition, 43); // Print a "+"
          stepnumber[selectedtrack]++;
          laststep[selectedtrack] = stepnumber[selectedtrack];
        }
      }
      // IF THIS TRACK IS A NOTE TRACK OR TRANSPOSE TRACK THEN WRITE A TIE
      if (tracktype[selectedtrack] == 0 || tracktype[selectedtrack] == 3) {
        if (button4State == 1 && button4StatePrev == 0 && button1State == 0 && button2State == 0 && button3State == 0 && button5State == 0) {
          button4StatePrev = 1;
        }
        if (button4State == 0 && button4StatePrev == 1) {
          button4StatePrev = 0;
          stepvalue[stepnumber[selectedtrack]][selectedtrack] = 129;
          lcd.setCursor(13, 0);
          lcd.print(stepnumber[selectedtrack] + 1);
          lcd.print(F(" "));
          printstep(stepLCDposition, 45); // Print a "-"
          stepnumber[selectedtrack]++;
          laststep[selectedtrack] = stepnumber[selectedtrack];
        }
      }

      /* Write a REST */
      if (button5State == 1  && button5StatePrev == 0 && button1State == 0 && button2State == 0 && button3State == 0 && button4State == 0) {
        stepvalue[stepnumber[selectedtrack]][selectedtrack] = 128;
        button5StatePrev = 1;
        //delay(2)
        lcd.setCursor(13, 0);
        lcd.print(stepnumber[selectedtrack] + 1);
        lcd.print(F(" "));
        printstep(stepLCDposition, 46); // Print a "."
        stepnumber[selectedtrack]++;
        laststep[selectedtrack] = stepnumber[selectedtrack];
      }
      if (button5State == 0 && button5StatePrev == 1) button5StatePrev = 0;

      /* One Step Backward with WRITE */
      if (button3State == 1 && button3StatePrev == 0 && writemode == 1 && stepnumber[selectedtrack] > 0 && button1State == 0 && button2State == 0 && button4State == 0 && button5State == 0) {
        button3StatePrev = 1;
        stepBack = 1;
      }
      if (button3State == 0 && button3StatePrev == 1) button3StatePrev = 0;

      if (stepBack == 1 ) {
        stepnumber[selectedtrack]--;
        laststep[selectedtrack] = stepnumber[selectedtrack];
        printstep(stepLCDposition - 1, 32); //print a " "
        lcd.setCursor(13, 0);
        lcd.print(stepnumber[selectedtrack] + 1);
        lcd.print(F(" "));
        stepBack = 0;
      }

      /* CONDITIONS FOR END WRITE */
      if (button2State == 1  && writemode == 1 && button1State == 0 && button3State == 0 && button4State == 0 && button5State == 0) { // END WRITE ON STOP
        button2StatePrev = 1;
        writemode = 0;
        clockInStatePrev = 0;
        //triggerreset();
        //midireset();
        clearscreen();
        pause[selectedtrack] = 1;
        if (runmode == 0 ) {
          lcd.setCursor(0, 0);
          lcd.print(F("STOP"));
        }
      }
      if (button2State == 0 && button2StatePrev == 1) button2StatePrev = 0;

      if (button1State == 1  && writemode == 1 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0) { // END WRITE ON START
        button1StatePrev = 1;
        writemode = 0;
        //center = (invalue[1]);
        //range = (invalue[2]);
        lcd.setCursor(0, 0);
        lcd.print(F("PLAY"));
        runmode = 1;
        clockInStatePrev = 0;
        triggerreset();
        pause[selectedtrack] = 2;
        //midireset();
        clearscreen();
        if (button1State == 0 && button1StatePrev == 1) button1StatePrev = 0;
      }

      if (stepnumber[selectedtrack] == maxSteps && writemode == 1 && button1State == 0 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0) { // END WRITE IF MAX TRACKLENGTH REACHED
        writemode = 0;
        clockInStatePrev = 0;
        pause[selectedtrack] = 1;
        triggerreset();
        //midireset();
        clearscreen();
      }
    }
    /* ------------------------ WRITE MODE ENDS HERE ------------------------ */
  } /* END OF SETUPMODE == 0*/


  /* ------------------------ PLAY MODE STARTS HERE ------------------------ */
  //if (runmode == 1 && writemode == 0) {
  if (runmode == 1) {

    /* SWITCHING THE PAUSE MODE */
    /* START WHILE PLAY MODE ACTIVATES PAUSE */
    if (button1State == 1 && button1StatePrev == 0 && pause[selectedtrack] == 0 && setupmode == 0 && comingfromsetupmode == 0 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0) { // ENTER PAUSE MODE
      button1StatePrev = 1;
      pause[selectedtrack] = 1;
    }
    if (button1State == 0 && button1StatePrev == 1) {
      button1StatePrev = 0;
    }

    /* START WHILE PAUSE MODE SETS TO WAIT-END-OF-PAUSE */
    if (button1State == 1 && button1StatePrev == 0 && pause[selectedtrack] == 1 && setupmode == 0 && comingfromsetupmode == 0 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0) { // ENTER WAIT PAUSE END MODE
      button1StatePrev = 1;
      pause[selectedtrack] = 2;
    }
    if (button1State == 0 && button1StatePrev == 1) {
      button1StatePrev = 0;
    }

    /* START WHILE WAIT-END-OF-PAUSE ACTIVATES PLAY IMMEDIATELY */
    if (button1State == 1 && button1StatePrev == 0  && pause[selectedtrack] == 2 && setupmode == 0 && comingfromsetupmode == 0 && button2State == 0 && button3State == 0 && button4State == 0 && button5State == 0) {
      pause[selectedtrack] = 0;
      button1StatePrev = 1;
    }
    if (button1State == 0 && button1StatePrev == 1) {
      button1StatePrev = 0;
    }

    /* STEP=0 WHILE WAIT-END-OF-PAUSE ACTIVATES PLAY AGAIN*/
    if (stepnumber[selectedtrack] == 0 && pause[selectedtrack] == 2) {
      //if (barClockCount < 4 && pause[selectedtrack] == 2) {
      pause[selectedtrack] = 0;
    }





    /* READ INCOMING MIDI CLOCK */
    if (MIDI.read()) {
      byte type = MIDI.getType();
      switch (type) {
        case midi::Clock:
          if (clockmode == 1) {
            if (midiclockcounter == 0) {
              if (clockmode == 1) digitalWrite(clockOutPin, HIGH);
            }

            if (midiclockcounter == ( 1 * clockfactor)) {
              if (clockmode == 1) digitalWrite(clockOutPin, LOW);
            }

            midiclockcounter++;
            if (midiclockcounter == (2 * clockfactor)) {
              midiclockcounter = 0;
            }
          }
          break;

        case midi::Stop:
          midiclockcounter = 0;
          runmode = 0;
          for ( playedtrack = 0 ; playedtrack < maxTracks ; playedtrack++ ) {
            if (tracktype[playedtrack] == 0 && noteOff[playedtrack] < 128 && noteOff[playedtrack] > 0) {
              MIDI.sendNoteOff(noteOff[playedtrack], velocity[playedtrack], outchannel[playedtrack]);
              noteOff[playedtrack] = 128;
            }
          }

          break;

        case midi::NoteOn:
          if ((notethrumode == 2 || notethrumode == 3) && (transposemodeglobal == 0 || (transposemodeglobal == 3 && button5State == 0) || transposemodeglobal > 3)) MIDI.sendNoteOn(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
          if (transposemodeglobal == 1 || transposemodeglobal == 2) {
            transposeall = MIDI.getData1();
            transposeall = transposeall - transposecenter;
            if (tracktype[selectedtrack] == 0) showtranspose(transposeall);
          }
          if (transposemodeglobal == 3 && button5State == 1) {
            transposeall = MIDI.getData1();
            transposeall = transposeall - transposecenter;
            if (tracktype[selectedtrack] == 0) showtranspose(transposeall);
          }


          for (i = 0; i < maxTracks; i++) {
            transpose[i] = transposeall;
          }
          break;

        case midi::NoteOff:
          if (notethrumode > 0) MIDI.sendNoteOff(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
          if (transposemodeglobal == 2) transposeall = 0;
          break;

        case midi::ControlChange:
          if ( controlthrumode == 1) MIDI.sendControlChange(MIDI.getData1(), MIDI.getData2(), outchannel[selectedtrack]);
          break;

        case midi::ProgramChange:
          if ( programthrumode == 1) MIDI.sendProgramChange(MIDI.getData1(), outchannel[selectedtrack]);
          break;

        case midi::PitchBend:
          if ( bendthrumode == 1) MIDI.sendPitchBend( (((MIDI.getData2() * 128) + MIDI.getData1() ) + 8192), outchannel[selectedtrack]);
          break;



      }
    } //END OF MIDI-READ


    barCount = 16;

    clockInState = !digitalRead(clockInPin);
    if (clockInState == 1 && clockInStatePrev == 0) {
      barClockCount++;
      if (barClockCount == barCount) {
        barClockCount = 0;
        if (pause[selectedtrack] == 2) {
          pause[selectedtrack] = 0;
        }
      }
      for ( playedtrack = 0 ; playedtrack <= maxTracks - 1 ; playedtrack++ ) {
        if ((clockdivider[playedtrack] - clockcounter[playedtrack]) % clockdivider[playedtrack] == 0) {
          ontrigger[playedtrack] = 1;
        }
        clockcounter[playedtrack]++;
      }
      clockInStatePrev = 1;
    }

    if (clockInState == 0 && clockInStatePrev == 1) {
      for ( playedtrack = 0 ; playedtrack <= maxTracks - 1 ; playedtrack++ ) {
        if ((clockdivider[playedtrack] - clockcounter[playedtrack]) % clockdivider[playedtrack] == 0) {
          offtrigger[playedtrack] = 1;
          clockcounter[playedtrack] = 0;
        }
      }
      clockInStatePrev = 0;
    }


    // First we prepare transpose and velocity data for the current step
    for ( playedtrack = 0 ; playedtrack < maxTracks ; playedtrack++ ) {
      if (writemode == 1 && playedtrack == selectedtrack) {
        // do nothing
      } else {
        // WITH ONTRIGGER IF TRACK IS VELOCITYTRACK OR TRANSPOSETRACK THEN LOAD THE VARIABLE "CURRENTVALUE" OF THIS TRACK
        if (ontrigger[playedtrack] == 1 && tracktype[playedtrack] >= 2) currentvalue[playedtrack] = stepvalue[stepnumber[playedtrack]][playedtrack];
      }
    }


    // We prepare the note data from velocity and transpose settings

    // First the global data

    // If globale transpose is a transposetrack:
    if (transposemodeglobal > 3) {
      if (currentvalue[transposemodeglobal - 4] < 128) transposeall = currentvalue[transposemodeglobal - 4]  - transposecenter;
      // the displayed track is a note track, then show the transposition (this should be better the transposition of the selected track)
      if (tracktype[selectedtrack] == 0 && writemode == 0) showtranspose(transposeall);
    }

    // If global velocity is a velocitytrack
    if (velocitymodeglobal > 2) {
      if (currentvalue[velocitymodeglobal - 3] < 128) velocityall = currentvalue[velocitymodeglobal - 2];
    }

    // Now the individual track data
    for ( playedtrack = 0 ; playedtrack < maxTracks ; playedtrack++ ) {
      if (writemode == 1 && playedtrack == selectedtrack) {
        // do nothing
      } else {
        if (ontrigger[playedtrack] == 1 && tracktype[playedtrack] == 0) {
          //SOURCE OF THIS TRACK IS A VELOCITY TRACK
          if (velocitysource[playedtrack] < maxTracks) {
            byte tempvelocity = (currentvalue[velocitysource[playedtrack]]);
            if (tempvelocity < 1) tempvelocity = 1;
            if (tempvelocity > 127) tempvelocity = 127;
            velocity[playedtrack] = tempvelocity;
          }
          //SOURCE OF THIS TRACK IS CONSTANT VELOCITY =16
          if (velocitysource[playedtrack] == maxTracks) velocity[playedtrack] = center;

          //SOURCE OF THIS TRACK IS RANDOM VELOCITY =17
          if (velocitysource[playedtrack] == (maxTracks + 1)) {
            byte tempvelocity = (random(1, 126) * (range / 128.0)) + ((2 * center) / 3);
            if (tempvelocity < 1) tempvelocity = 1;
            if (tempvelocity > 127) tempvelocity = 127;
            velocity[playedtrack] = tempvelocity;
          }
          //SOURCE OF THIS TRACK IS GLOBAL VELOCITY =18
          //CONSTANT
          if ((velocitysource[playedtrack] == (maxTracks + 2) ) && velocitymodeglobal == 0) {
            velocity[playedtrack] = center;
          }
          //RANDOM
          if ((velocitysource[playedtrack] == (maxTracks + 2) ) && velocitymodeglobal == 1) {
            byte tempvelocity = (random(1, 126) * (range / 128.0)) + ((2 * center) / 3);
            if (tempvelocity < 1) tempvelocity = 1;
            if (tempvelocity > 127) tempvelocity = 127;
            velocity[playedtrack] = tempvelocity;
          }

          //TRANSPOSE
          //TRANSPOSE TRACK
          if (transposesource[playedtrack] < maxTracks && transposesource[playedtrack] != playedtrack) {
            if (currentvalue[transposesource[playedtrack]] < 128 && currentvalue[transposesource[playedtrack]] > 0) transpose[playedtrack] = currentvalue[transposesource[playedtrack]] - transposecenter;
          }
          // NO TRANSPOSE
          if (transposesource[playedtrack] == maxTracks) transpose[playedtrack] = 0;
          // GLOBAL TRANSPOSE
          if (transposesource[playedtrack] == maxTracks + 1) transpose[playedtrack] = transposeall;
        }
      }
    }

    // We send all ControlChange Date befor we send the notes, so the sound changes get in effect, before the notes invoke the sound
    for ( playedtrack = 0 ; playedtrack <= maxTracks - 1 ; playedtrack++ ) {
      if (writemode == 1 && playedtrack == selectedtrack) {
        // do nothing
      } else {
        if (ontrigger[playedtrack] == 1) {
          if (tracktype[playedtrack] == 1  && pause[playedtrack] == 0 && stepvalue[stepnumber[playedtrack]][playedtrack] < 128) MIDI.sendControlChange(ccnumber[playedtrack], stepvalue[stepnumber[playedtrack]][playedtrack], outchannel[playedtrack]);
        }
      }
    }

    for ( playedtrack = 0 ; playedtrack <= maxTracks - 1 ; playedtrack++ ) {
      if (writemode == 1 && playedtrack == selectedtrack) {
        //do nothing
      } else {
        if (ontrigger[playedtrack] == 1) {
          if (noteoffmode == 0 && stepvalue[stepnumber[playedtrack]][playedtrack] <= 128 && noteOff[playedtrack] > 0)  {
            MIDI.sendNoteOff(noteOff[playedtrack], velocity[playedtrack], outchannel[playedtrack]);
            noteOff[playedtrack] = 0;
          }
          //transpose[playedtrack] = 0;
          //lcd.setCursor(5, 0);
          //lcd.print(transpose[selectedtrack]);
          //lcd.print(transposecenter);
          //lcd.setCursor(9, 0);
          //lcd.print(stepvalue[stepnumber[selectedtrack]][selectedtrack]);
          if (tracktype[playedtrack] == 0  && pause[playedtrack] == 0 && stepvalue[stepnumber[playedtrack]][playedtrack] < 128) MIDI.sendNoteOn(stepvalue[stepnumber[playedtrack]][playedtrack] + transpose[playedtrack], velocity[playedtrack], outchannel[playedtrack]);
          if (tracktype[playedtrack] == 0  && pause[playedtrack] == 0 && stepvalue[stepnumber[playedtrack]][playedtrack] < 128) noteOff[playedtrack] = stepvalue[stepnumber[playedtrack]][playedtrack] + transpose[playedtrack];
          ontrigger[playedtrack] = 0;
        }
      }
    }




    /*PRINT THE STEP SYMBOLS*/
    stepLCDposition = stepnumber[selectedtrack] % 16;
    if (stepvalue[stepnumber[selectedtrack]][selectedtrack] < 128) {
      printstep(stepLCDposition, 43); //print a "+"
    }
    if (stepvalue[stepnumber[selectedtrack]][selectedtrack] == 128) {
      printstep(stepLCDposition, 46); //print a "."
    }
    if (stepvalue[stepnumber[selectedtrack]][selectedtrack] == 129) {
      printstep(stepLCDposition, 45); //print a "-"
    }







    for ( playedtrack = 0 ; playedtrack <= maxTracks - 1 ; playedtrack++ ) {
      if (writemode == 1 && playedtrack == selectedtrack) {
        // do nothing
      } else {
        if (offtrigger[playedtrack] == 1) {
          if (tracktype[playedtrack] == 0 && noteOff[playedtrack] < 128 && noteOff[playedtrack] > 0 && noteoffmode == 1) {
            MIDI.sendNoteOff(noteOff[playedtrack], velocity[playedtrack], outchannel[playedtrack]);
            noteOff[playedtrack] = 0;
          }
          offtrigger[playedtrack] = 0;
          stepnumber[playedtrack]++;
          if (stepnumber[playedtrack] == (laststep[playedtrack])) stepnumber[playedtrack] = 0;
        }
      }
    }
  } //END OF RUNMODE
  /* ------------------------ PLAY MODE ENDS HERE ------------------------ */


  /*========HERE BEGINS THE SETUP SECTION========*/
  if (setupmode == 1 ) { //SETUP

    MIDI.read();
    lcd.setCursor(0, 1);
    if (edit == 0) lcd.print(F(" "));
    if (edit == 1) lcd.print(F("!"));

    /* STOP+PLAY exits the setup */
    if ( button1State == 1 && button2State == 1 && edit == 0 && button3State == 0 && button4State == 0 && button5State == 0) {
      comingfromsetupmode = 2;
      setupmode = 0;
    }

    /* START  PRESSED + BUTTON F1 DECREASE SELECTED TRACK */
    if (button1State == 1 && button4State == 1 && button4StatePrev == 0 && saveMulti == 1 && button2State == 0 && button3State == 0 && button5State == 0) {
      button4StatePrev = 1;
      selectedtrack--;
      if (selectedtrack == 255) selectedtrack = maxTracks - 1;
    }
    if (button4State == 0 && button4StatePrev == 1) button4StatePrev = 0;

    /* START PRESSED + BUTTON F2 INCREASES SELECTED TRACK */
    if (button1State == 1 && button5State == 1 && button5StatePrev == 0 && writemode == 0 && saveMulti == 1 && button2State == 0 && button3State == 0 && button4State == 0) {
      button5StatePrev = 1;
      selectedtrack++;
      if (selectedtrack == maxTracks) selectedtrack = 0;
    }
    if (button5State == 0 && button5StatePrev == 1) button5StatePrev = 0;


    /*
      Stepping thru the setup menu
    */
    if (edit == 0) {
      menuPrev = menu;
      menu = getDataValue(menu, 25, 0, 0, 1);
      if (menu != menuPrev) clearscreen();

      if (button3State == 1 && button3StatePrev == 0 && menu != 16 && button1State == 0 && button2State == 0 && button4State == 0 && button5State == 0) { // ENTER EDIT MODE
        button3StatePrev = 1;
        if (menu == 15 && runmode == 1) {
          lcd.setCursor(1, 1);
          lcd.print(F("Not while Play"));
        } else {
          edit = 1;
        }
      }
    }

    do_menu();

    if (edit == 1) {
      if (button3State == 1 && button3StatePrev == 0 && ( menu < 16 || menu > 20 ) && button1State == 0 && button2State == 0 && button4State == 0 && button5State == 0) {
        button3StatePrev = 1;
        edit = 0;
        if (saveMulti == 0) EEPROM.update(saveMemory, saveValue);
        if (saveMulti == 1) {
          for (i = 0; i < maxTracks; i++) {
            EEPROM.update(saveMemory + i, saveValueMulti[i]);
          }
        }
      }
    }

    if (button3State == 0 && button3StatePrev == 1) {
      button3StatePrev = 0;
    }

    /* AGAIN SOME DEBUGGING OUTPUT
      lcd.setCursor(9, 0);
      lcd.print(button3State);
      lcd.print(button3StatePrev);
      lcd.print(edit);
      /**/
  }


}
/* END OF LOOP */

void do_menu() {

  /*
     Transpose,Velocity,NoteOffMode,InChannel,OutChannel[t],tracktype[t],ccnumber[t],velocitysource[t],transposesource[t],ClockDivider[t],PlayOnStart[t],NoteThru,ControlThru,ProgrammThru,BendThru,OtherChannelThru,CcEditMode,TrimmPotis,factoryreset,LoadSequence,SaveSequence,ClockSelect,TransposeCenter
     0        1         2           3         4             5            6           7                8                 9               10             11       12          13          ,14      ,15              ,16        ,17        ,18          ,19          ,20          ,21         ,22
  */



  switch (menu) {
    // getDataValue(int myvalue, int maxValue, int minValue, int incten, int runover)
    case 0 : //GLOBAL TRANSPOSE
      if (edit == 1) transposemodeglobal = getDataValue(transposemodeglobal, 3 + maxTracks, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("TRANSPOSE    All"));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (transposemodeglobal == 0) lcd.print(F("Off           "));
        if (transposemodeglobal == 1) lcd.print(F("Continous     "));
        if (transposemodeglobal == 2) lcd.print(F("Momentary     "));
        if (transposemodeglobal == 3) lcd.print(F("ButtonEnabled "));
        if (transposemodeglobal >= 4) {
          lcd.print(F("Track "));
          lcd.print(transposemodeglobal - 3);
          lcd.print(F("         "));
        }
      }
      saveMemory = 8;
      saveValue = transposemodeglobal;
      break;

    case 1 : //GLOBAL VELOCITY
      if (edit == 1) velocitymodeglobal = getDataValue(velocitymodeglobal, 1 + maxTracks, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("VELOCITY     All"));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (velocitymodeglobal == 0) lcd.print(F("Constant (CV1)  "));
        if (velocitymodeglobal == 1) lcd.print(F("Random   (CV2)  "));
        if (velocitymodeglobal >= 2) {
          lcd.print(F("Track "));
          lcd.print(velocitymodeglobal - 1);
          lcd.print(F("         "));
        }
      }
      saveMemory = 8;
      saveValue = transposemodeglobal;
      break;

    case 2: //NOTEOFFMODE
      if (edit == 1) noteoffmode = getDataValue(noteoffmode, 2, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("NOTEOFF-MODE All"));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (noteoffmode == 0) lcd.print(F("Legato       "));
        if (noteoffmode == 1) lcd.print(F("Gated        "));
        if (noteoffmode == 2) lcd.print(F("CV-ctrl      "));
      }
      saveMemory = 9;
      saveValue = noteoffmode;
      break;


    case 3://INCHANNEL
      if (edit == 1) inchannel = getDataValue(inchannel, 16, 1, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("IN CHANNEL   All"));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        lcd.print(inchannel);
        lcd.print(F("      "));
      }
      saveMemory = 5;
      saveValue = inchannel;
      MIDI.setInputChannel(inchannel);
      break;




    case 4://OUTCHANNEL T
      if (edit == 1) outchannel[selectedtrack] = getDataValue(outchannel[selectedtrack], 16, 1, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("OUT CHANNEL "));
      lcd.print(F("Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        lcd.print(outchannel[selectedtrack]);
        lcd.print(F("  "));
      }
      saveMemory = 32 + (3 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = outchannel[i];
      }
      break;

    case 5: //TRACKTYPE T
      if (edit == 1) tracktype[selectedtrack] = getDataValue(tracktype[selectedtrack], 3, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("TRACKTYPE   "));
      lcd.print(F("Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (tracktype[selectedtrack] == 0) lcd.print(F("Note           "));
        if (tracktype[selectedtrack] == 1) lcd.print(F("CC             "));
        if (tracktype[selectedtrack] == 2) lcd.print(F("Velocity       "));
        if (tracktype[selectedtrack] == 3) lcd.print(F("Transpose      "));
      }
      saveMemory = 32 + (0 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = tracktype[i];
      }
      break;

    case 6://CC-NUMBER T
      if (edit == 1) ccnumber[selectedtrack] = getDataValue(ccnumber[selectedtrack], 127, 1, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("CC-NUMBER   "));
      lcd.print(F("Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));

      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (tracktype[selectedtrack] != 1) {
          lcd.print(F("Not a CC-Track"));
        } else {
          lcd.print(ccnumber[selectedtrack]);
          lcd.print(F("             "));
        }
      }
      saveMemory = 32 + (1 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = ccnumber[i];
      }
      break;

    case 7://velocitysource T
      if (edit == 1) velocitysource[selectedtrack] = getDataValue(velocitysource[selectedtrack], maxTracks + 2, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("VEL.SOURCE  Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (tracktype[selectedtrack] != 0) {
          lcd.print(F("Not a note-trk "));
        } else {
          if (velocitysource[selectedtrack] == maxTracks)     lcd.print(F("Constant (CV1)  "));
          if (velocitysource[selectedtrack] == maxTracks + 1) lcd.print(F("Random   (CV2)  "));
          if (velocitysource[selectedtrack] == maxTracks + 2) lcd.print(F("Global Velocity "));
          if (velocitysource[selectedtrack] < maxTracks) {
            lcd.print(F("Track "));
            lcd.print(velocitysource[selectedtrack] + 1);
            if (tracktype[velocitysource[selectedtrack]] != 2) {
              lcd.print(F(" (?)          "));
            } else {
              lcd.print(F("              "));
            }
          }
        }
      }
      saveMemory = 32 + (4 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = velocitysource[i];
      }
      break;

    case 8://transposesource T
      if (edit == 1) transposesource[selectedtrack] = getDataValue(transposesource[selectedtrack], maxTracks + 1, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("TRANS.SRC   Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (tracktype[selectedtrack] != 0) lcd.print(F("Not a note-trk "));
        if (transposesource[selectedtrack] == maxTracks)     lcd.print(F("No Transpose    "));
        if (transposesource[selectedtrack] == maxTracks + 1) lcd.print(F("Global Transp.  "));
        if (transposesource[selectedtrack] == selectedtrack) lcd.print(F("Invalid         "));
        if (transposesource[selectedtrack] < maxTracks) {
          lcd.print(F("Track "));
          lcd.print(transposesource[selectedtrack] + 1);
          if (tracktype[transposesource[selectedtrack]] != 3) {
            lcd.print(F(" (?)          "));
          } else {
            lcd.print(F("              "));
          }
        }
      }

      saveMemory = 32 + (5 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = transposesource[i];
      }
      break;

    case 9://CLOCKDIVIDER T
      if (edit == 1) {
        clockdivider[selectedtrack] = getDataValue(clockdivider[selectedtrack], 128, 1, 0, 1);
      }
      lcd.setCursor(0, 0);
      lcd.print(F("CLOCKDIVIS. "));
      lcd.print(F("Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        lcd.print(clockdivider[selectedtrack]);
        lcd.print(F("  "));
      }

      saveMemory = 32 + (2 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = clockdivider[i];
      }
      break;

    case 10: //START BEHAVIOUR T
      if (edit == 1) playonstart[selectedtrack] = getDataValue(playonstart[selectedtrack], 1, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("PLAYONSTART Tr"));
      lcd.print(selectedtrack + 1);
      lcd.print(F(" "));
      if ( displaymode == 0) {
        lcd.setCursor(1, 1);
        if (playonstart[selectedtrack] == 0) lcd.print(F("Immediate start"));
        if (playonstart[selectedtrack] == 1) lcd.print(F("Obey pause mode"));
      }
      saveMemory = 32 + (6 * maxTracks);
      saveMulti = 1;
      for ( i = 0; i < maxTracks; i++ ) {
        saveValueMulti[i] = playonstart[i];
      }
      break;



    case 11: //NOTETHRU
      notethrumode = getDataValue(notethrumode, 3, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("NOTE-THRU-MODE"));
      if (displaymode == 0) {
        lcd.setCursor(1, 1);
        if (notethrumode == 0) lcd.print(F("Off          "));
        if (notethrumode == 1) lcd.print(F("Record       "));
        if (notethrumode == 2) lcd.print(F("Play         "));
        if (notethrumode == 3) lcd.print(F("Rec+Play     "));
        if (notethrumode == 4) lcd.print(F("PlayTriggered")); // not in use and not selectable
      }
      saveMemory = 11;
      saveValue = notethrumode;
      break;

    case 12: //CONTROLTHRU
      controlthrumode = getDataValue(controlthrumode, 1, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("CTRL-THRU-MODE"));
      if (displaymode == 0) {
        lcd.setCursor(1, 1);
        if (controlthrumode == 0) lcd.print(F("Off"));
        if (controlthrumode == 1) lcd.print(F("On "));
      }
      saveMemory = 12;
      saveValue = controlthrumode;
      break;

    case 13: //PROGRAM
      programthrumode = getDataValue(programthrumode, 1, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("PRG-THRU-MODE"));
      if (displaymode == 0) {
        lcd.setCursor(1, 1);
        if (programthrumode == 0) lcd.print(F("Off"));
        if (programthrumode == 1) lcd.print(F("On "));
      }
      saveMemory = 13;
      saveValue = programthrumode;
      break;

    case 14: //BENDER
      bendthrumode = getDataValue(bendthrumode, 1, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("BEND-THRU-MODE"));
      if (displaymode == 0) {
        lcd.setCursor(1, 1);
        if (bendthrumode == 0) lcd.print(F("Off"));
        if (bendthrumode == 1) lcd.print(F("On "));
      }
      saveMemory = 14;
      saveValue = bendthrumode;
      break;

    case 15: //OTHER-CHANNELS-THRU
      otherthrumode = getDataValue(otherthrumode, 1, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("OTHER-THRU-MODE"));
      if (displaymode == 0) {
        lcd.setCursor(1, 1);
        if (otherthrumode == 0) lcd.print(F("Off"));
        if (otherthrumode == 1) lcd.print(F("On "));
      }
      saveMemory = 15;
      saveValue = otherthrumode;
      if (otherthrumode == 0) {
        MIDI.setThruFilterMode(0);
        MIDI.turnThruOff();
      }
      if (otherthrumode == 1) {
        MIDI.setThruFilterMode(3);
        MIDI.turnThruOn(3);
      }
      break;

    case 16: //COPY PASTE
      if (edit == 0) {
        lcd.setCursor(0, 0);
        lcd.print(F("COPY FROM   "));
        lcd.print(F("Tr"));
        lcd.print(selectedtrack + 1);
        lcd.print(F(" "));

        lcd.setCursor(1, 1);
        lcd.print(F("PRESS WRITE   "));
        if (button3State == 1 && button3StatePrev == 0) {
          button3StatePrev = 1;
          EEPROM.write((20 + 0), tracktype[selectedtrack]);
          EEPROM.write((20 + 1), ccnumber[selectedtrack]);
          EEPROM.write((20 + 2), clockdivider[selectedtrack]);
          EEPROM.write((20 + 3), outchannel[selectedtrack]);
          EEPROM.write((20 + 4), velocitysource[selectedtrack]);
          EEPROM.write((20 + 5), transposesource[selectedtrack]);
          EEPROM.write((20 + 6), playonstart[selectedtrack]);
          delay(20);
          clearscreen();
          for (i = 0; i < 6; i++) {
            lcd.setCursor(0, 0);
            lcd.print(i);
            lcd.print(F(" "));
            lcd.print(EEPROM.read(20 + i));
            lcd.print(F("              "));
            delay(80);
          }
          button2StatePrev = 0;
          edit = 1;
        }
        if (button3State == 0 && button3StatePrev == 1) {
          button3StatePrev = 0;
        }
      }
      /* */
      if (edit == 1) {
        lcd.setCursor(0, 0);
        lcd.print(F("PASTE TO    "));
        lcd.print(F("Tr"));
        lcd.print(selectedtrack + 1);
        lcd.print(F(" "));
        lcd.setCursor(1, 1);
        lcd.print(F("WRITE or STOP "));
        if (button3State == 1 && button3StatePrev == 0) {
          button3StatePrev = 1;
          tracktype[selectedtrack]       = EEPROM.read((20 + 0));
          ccnumber[selectedtrack]        = EEPROM.read((20 + 1));
          clockdivider[selectedtrack]    = EEPROM.read((20 + 2));
          outchannel[selectedtrack]      = EEPROM.read((20 + 3));
          velocitysource[selectedtrack]  = EEPROM.read((20 + 4));
          transposesource[selectedtrack] = EEPROM.read((20 + 5));
          playonstart[selectedtrack]     = EEPROM.read((20 + 6));
          delay(20);
          clearscreen();
          for (i = 0; i < 6; i++) {
            lcd.setCursor(0, 0);
            lcd.print(i);
            lcd.print(F(" "));
            lcd.print(EEPROM.read(20 + i));
            lcd.print(F("              "));
            delay(80);
          }
        }
        if (button3State == 0 && button3StatePrev == 1) {
          button3StatePrev = 0;
        }
        if (button2State == 1 && button2StatePrev == 0) {
          button2StatePrev = 1;
          edit = 0;
        }
        if (button2State == 0 && button2StatePrev == 1) {
          button2StatePrev = 0;
        }

      }

      // */
      break;


    case 17: //POTENTIOMETER ADJUSTMENT
      if (edit == 0) {
        lcd.setCursor(0, 0);
        lcd.print(F("ADJUST POTI-CVs"));
        lcd.setCursor(1, 1);
        lcd.print(F("PRESS WRITE"));
      }
      if (edit == 1) {
        lcd.setCursor(0, 0);
        lcd.print(F("SET POTIS TO MAX"));
        lcd.setCursor(1, 1);
        lcd.print(F("THEN PRESS WRITE"));
        if (button3State == 1 && button3StatePrev == 0) {
          button3StatePrev = 1;
          clearscreen();
          adjuststep = 0;
          adjustpotis();
          edit = 0;
          clearscreen();
        }
        if (button3State == 0 && button3StatePrev == 1) button3StatePrev = 0;
      }
      break;

    case 18: //FACTORY RESET
      if (edit == 0) {
        lcd.setCursor(0, 0);
        lcd.print(F("FACTORY RESET"));
        lcd.setCursor(1, 1);
        lcd.print(F("PRESS 2xWRITE"));
      }
      if (edit == 1) {
        lcd.setCursor(0, 0);
        lcd.print(F("RESET:PRESS WRITE"));
        lcd.setCursor(1, 1);
        lcd.print(F("STOP TO CANCEL"));
        if (button3State == 1 && button3StatePrev == 0) {
          runmode = 0;
          button3StatePrev = 1;
          eepromfactoryreset();
          clearscreen();
          loadsettings();
          loadfactorysequence();
          //savesequence();
          //delay(10);
          edit = 0;
          clearscreen();
          menu = 0;
        }
        if (button3State == 0 && button3StatePrev == 1) button3StatePrev = 0;
        if (button2State == 1 && button2StatePrev == 0 && menu > 0) {
          button2StatePrev = 1;
        }
        if (button2State == 0 && button2StatePrev == 1 && menu > 0) {
          button2StatePrev = 0;
          clearscreen();
          edit = 0;
        }
      }
      break;

    case 19: //LOAD SEQUENCE
      if (edit == 0) {
        lcd.setCursor(0, 0);
        lcd.print(F("LOAD PATTERN"));
        lcd.setCursor(1, 1);
        lcd.print(F("PRESS 2xWRITE"));
      }
      if (edit == 1) {
        lcd.setCursor(0, 0);
        lcd.print(F("LOAD PRESS WRITE"));
        lcd.setCursor(1, 1);
        lcd.print(F("STOP FOR CANCEL"));
        if (button3State == 1 && button3StatePrev == 0) {
          runmode = 0;
          button3StatePrev = 1;
          loadsequence();
          //delay(10);
          edit = 0;
          clearscreen();
          setupmode = 0;
        }
        if (button3State == 0 && button3StatePrev == 1) {
          button3StatePrev = 0;
        }
        if (button2State == 1 && button2StatePrev == 0 && menu > 0) {
          button2StatePrev = 1;
        }
        if (button2State == 0 && button2StatePrev == 1 && menu > 0) {
          button2StatePrev = 0;
          clearscreen();
          edit = 0;
        }
      }
      break;

    case 20: //SAVE SEQUENCE
      if (edit == 0) {
        lcd.setCursor(0, 0);
        lcd.print(F("SAVE PATTERN"));
        lcd.setCursor(1, 1);
        lcd.print(F("PRESS 2xWRITE"));
      }
      if (edit == 1) {
        lcd.setCursor(0, 0);
        lcd.print(F("SAVE PRESS WRITE"));
        lcd.setCursor(1, 1);
        lcd.print(F("STOP FOR CANCEL"));
        if (button3State == 1 && button3StatePrev == 0) {
          button3StatePrev = 1;
          savesequence();
          edit = 0;
          lcd.setCursor(1, 1);
          lcd.print(F("SAVE DONE    "));
          if (runmode == 0) delay(500);
          clearscreen();
          setupmode = 0;
        }
        if (button3State == 0 && button3StatePrev == 1) {
          button3StatePrev = 0;
        }
        if (button2State == 1 && button2StatePrev == 0 && menu > 0) {
          button2StatePrev = 1;
        }
        if (button2State == 0 && button2StatePrev == 1 && menu > 0) {
          button2StatePrev = 0;
          clearscreen();
          edit = 0;
        }
      }
      break;

    case 21: //CLOCK SELECT
      clockmode = getDataValue(clockmode, 2, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("CLOCK SOURCE"));
      lcd.setCursor(1, 1);
      if (clockmode == 0) lcd.print(F("Internal       "));
      if (clockmode == 1) lcd.print(F("MIDI Clock     "));
      if (clockmode == 2) lcd.print(F("VMC Clock      "));
      saveMemory = 6;
      saveValue = clockmode;

      saveMulti = 0;
      break;

    case 22: //START/STOP CONTROL
      startstopmode = getDataValue(startstopmode, 2, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("START/STOP SRC"));
      lcd.setCursor(1, 1);
      if (startstopmode == 0) lcd.print(F("Internal only  "));
      if (startstopmode == 1) lcd.print(F("MIDI START/STOP"));
      if (startstopmode == 2) lcd.print(F("VMC START/STOP "));
      saveMemory = 18;
      saveValue = startstopmode;
      saveMulti = 0;
      break;

    case 23: //SEND CLOCK
      sendclock = getDataValue(sendclock, 3, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("SEND CLOCK TO..."));
      lcd.setCursor(1, 1);
      if (sendclock == 0) lcd.print(F("...none        "));
      if (sendclock == 1) lcd.print(F("...MIDI        "));
      if (sendclock == 2) lcd.print(F("...MIDI + VMC  "));
      if (sendclock == 3) lcd.print(F("...VMC         "));
      saveMemory = 19;
      saveValue = sendclock;
      saveMulti = 0;
      break;

    case 24: //CLOCK DIVIDER
      inclockdivider = getDataValue(inclockdivider, 6, 0, 0, 1);
      lcd.setCursor(0, 0);
      lcd.print(F("CLOCK DIVIDER"));
      lcd.setCursor(1, 1);


      if (inclockdivider == 0) {
        lcd.print(F("1/16          "));
      }
      if (inclockdivider == 1) {
        lcd.print(F("1/8           "));
      }
      if (inclockdivider == 2) {
        lcd.print(F("1/4           "));
      }
      if (inclockdivider == 3) {
        lcd.print(F("1/1           "));
      }
      if (inclockdivider == 4) {
        lcd.print(F("2/1           "));
      }
      if (inclockdivider == 5) {
        lcd.print(F("1/12          "));
      }
      if (inclockdivider == 6) {
        lcd.print(F("1/24          "));
      }
      setClockFactor();
      saveMemory = 17;
      saveValue = inclockdivider;
      saveMulti = 0;
      break;


    case 25: //TRANSPOSE CENTER NOTE
      lcd.setCursor(0, 0);
      lcd.print(F("TRANSP.CENTER"));
      lcd.setCursor(1, 1);
      lcd.print(transposecenter);
      lcd.print(F(" or send note  "));

      if (edit == 1) {
        if (button4State == 0 && button5State == 0) {
          if (MIDI.read()) {
            byte type = MIDI.getType() ;
            if (type == midi::NoteOn && MIDI.getChannel() == inchannel) {
              transposecenter = MIDI.getData1();
            }
          }
        }
        transposecenter = getDataValue(transposecenter, 127, 1, 0, 1);
      }
      saveMemory = 7;
      saveValue = transposecenter;
      break;
  } // END OF SWITCH:CASE

} // END OF DO_MENU()


int getDataValue(int myvalue, int maxValue, int minValue, int incten, int runover) {
  if (button1State == 0 && button4State == 1 && button4StatePrev == 0) {
    button4StatePrev = 1;
    myvalue--;
  }
  if (button4State == 0 && button4StatePrev == 1) {
    button4StatePrev = 0;
  }

  if (button1State == 0 && button5State == 1 && button5StatePrev == 0) { // SCROLL THRU MENU UP
    button5StatePrev = 1;
    myvalue++;
  }
  if (button5State == 0 && button5StatePrev == 1) {
    button5StatePrev = 0;
  }

  if (edit > 0 && button2State == 1 && (invalue[2] != invaluePrev[2])) myvalue = invalue[2] /  (128 / maxValue);

  if (runover == 0 && myvalue > maxValue) myvalue = maxValue;
  if (runover == 0 && myvalue < minValue) myvalue = minValue;
  if (runover == 1 && myvalue > maxValue) myvalue = minValue;
  if (runover == 1 && myvalue < minValue) myvalue = maxValue;
  return myvalue;
}

void printstep (byte step, char type) {
  if (setupmode == 0) {
    if (step == 0) {
      lcd.setCursor(1, 1);
      lcd.print(F("               "));
    }
    lcd.setCursor(step, 1);
    lcd.print(type);
  }
}


void showtranspose(int transpose) {
  lcd.setCursor(4, 0);
  if (setupmode == 0) {
    if (transpose < 0) {
      lcd.print(F(" Trns"));
      lcd.setCursor(9, 0);
      lcd.print(transpose);
      if (transpose > -10) {
        lcd.setCursor(11, 0);
        lcd.print(F(" "));
      }
    }
    if (transpose == 0) lcd.print(F("        "));
    if (transpose > 0) {
      lcd.print(F(" Trns+"));
      lcd.setCursor(10, 0);
      lcd.print(transpose);
      if (transpose < 10) {
        lcd.setCursor(11, 0);
        lcd.print(F(" "));
      }
    }
  }
}



void clearscreen() {
  lcd.setCursor(0, 0);
  lcd.print(F("                "));
  lcd.setCursor(0, 1);
  lcd.print(F("                "));
}




void timerIsr()
{
  if (clockmode == 0) {
    if (dinclockcounter == 0) {
      if (sendclock == 2 || sendclock == 3) digitalWrite(dinClockPin, HIGH);
      if (sendclock == 1 || sendclock == 2) {
        using namespace midi;
        MIDI.sendRealTime(Clock);
      }
    }
    if (intclockcounter == 6) {
      digitalWrite(clockOutPin, HIGH);
    }

    if (intclockcounter == (8 * clockfactor)) {
      if (clockmode == 0) digitalWrite(clockOutPin, LOW);
    }
    if (dinclockcounter == 4) {
      if (sendclock == 2 || sendclock == 3) digitalWrite(dinClockPin, LOW);
    }

    // We send the start command inbetween two clock commands and not if the DIN-clock still is high
    if (dinclockcounter == 5) {
      if (startFlag == 1) {
        runmode = 1;
        if (sendclock == 1 || sendclock == 2) {
          using namespace midi;
          MIDI.sendRealTime(Start);
        }
        if (sendclock == 2 || sendclock == 3) {
          digitalWrite(dinGatePin, HIGH);
        }
        startFlag = 0;
      }
    }

    intclockcounter ++;
    dinclockcounter ++;
    if (intclockcounter == (16 * clockfactor)) {
      intclockcounter = 0;
    }
    if (dinclockcounter == 8) {
      dinclockcounter = 0;
    }
  }

}




void setClockFactor() {
  if (inclockdivider == 0) { // 1/16
    clockfactor = 3;
  }
  if (inclockdivider == 1) {// 1/8
    clockfactor = 6;
  }
  if (inclockdivider == 2) {// 1/4
    clockfactor = 12;
  }
  if (inclockdivider == 3) {// 1/1
    clockfactor = 48;
  }
  if (inclockdivider == 4) {// 2/1
    clockfactor = 96;
  }
  if (inclockdivider == 5) {// 1/12
    clockfactor = 4;
  }
  if (inclockdivider == 6) {// 1/24
    clockfactor = 2;
  }
  if (intclockcounter >= (16 * clockfactor)) {
    intclockcounter = 0;
    dinclockcounter = 0;
  }
  if (midiclockcounter >= (2 * clockfactor)) {
    midiclockcounter = 0;
  }
}





void savesequence() {
  lcd.setCursor(1, 1);
  lcd.print(F("               "));
  byte track;
  for (track = 0; track < maxTracks; track++) {
    lcd.setCursor(track, 1);
    lcd.print(F("."));
    delay(50);
    for ( stepnumber[track] = 0; stepnumber[track] < maxSteps; stepnumber[track]++) {
      uint16_t address = 32 + (9 * maxTracks) + (track * maxSteps) + stepnumber[track];
      EEPROM.update(address, stepvalue[stepnumber[track]][track]);
    }
  }

  for (track = 0; track <= maxTracks - 1; track++) {
    uint16_t address = 32 + (7 * maxTracks) + track;
    EEPROM.update(address, laststep[track]);
  }

//  lcd.setCursor(0, 1);
//  lcd.print(F("SAVE DONE       "));
//  delay(500);
  clearscreen();
}

void loadsequence() {
  byte track;
  for (track = 0; track < maxTracks; track++) {
    lcd.setCursor(15 - track, 1);
    lcd.print(F("."));
    delay(50);
    for ( stepnumber[track] = 0; stepnumber[track] < maxSteps; stepnumber[track]++) {
      uint16_t address = 32 + (9 * maxTracks) + (track * maxSteps) + stepnumber[track];
      stepvalue[stepnumber[track]][track] = EEPROM.read(address);
    }
  }
  for (track = 0; track < maxTracks; track++) {
    uint16_t address = 32 + (7 * maxTracks) + track;
    laststep[track] = EEPROM.read(address);
  }

//  lcd.setCursor(0, 1);
//  lcd.print(F("LOAD DONE       "));
//  delay(500);
  clearscreen();
}

void midireset() {
  byte track;
  byte i = 0;
  for (track = 0; track < maxTracks; track++) {
    for (i = 0; i <= 126 ; i++) {
      MIDI.sendNoteOff(i, 0, outchannel[track]);
      //  delay(1);
    }
    MIDI.sendControlChange(123, 0, outchannel[track]);
  }
}

void adjustpotis() {
  thisRead[0] = analogRead(input[0]);
  thisRead[1] = analogRead(input[1]);
  thisRead[2] = analogRead(input[2]);
  i = 0;
  while (i < 3) {
    invalueOffset[i] = 0;
    while ((thisRead[i] + invalueOffset[i]) != 1023) {
      thisRead[i] = analogRead(input[i]) + 0.5 ;
      if ((thisRead[i] + invalueOffset[i]) < 1023) invalueOffset[i]++;
      if ((thisRead[i] + invalueOffset[i]) > 1023) invalueOffset[i]--;
      lcd.setCursor(0, 0);
      lcd.print(invalueOffset[i]);
      lcd.print(" ");
      lcd.setCursor(7, 0);
      if (i == 0) lcd.print(F("Tempo"));
      if (i == 1) lcd.print(F("CV1  "));
      if (i == 2) lcd.print(F("CV2  "));
    }
    if ((thisRead[i] + invalueOffset[i]) == 1023 ) {
      lcd.setCursor(0, 1);
      lcd.print(F("Good!  "));
      EEPROM.update(i, invalueOffset[i]);
      delay(500);
      clearscreen();
      i++;
    }
  }
  clearscreen();
  i = 0;
}

void triggerreset() {
  for (i = 0; i <= maxTracks - 1; i++) {
    ontrigger[i] = 0;
    offtrigger[i] = 0;
  }
}

void loadsettings() {
  /* READ GENERIC PARATMETERS FROM EEPROM */
  //outchannelall   = EEPROM.read(4);
  inchannel       = EEPROM.read(5);
  //clockmode       = EEPROM.read(6);
  clockmode       = 0; // we always set the clock to internal clock on resets
  transposecenter = EEPROM.read(7);
  transposemodeglobal   = EEPROM.read(8);
  //notelength      = EEPROM.read(10);
  notethrumode    = EEPROM.read(11);
  controlthrumode = EEPROM.read(12);
  programthrumode = EEPROM.read(13);
  bendthrumode    = EEPROM.read(14);
  otherthrumode   = EEPROM.read(15);
  velocitymodeglobal    = EEPROM.read(16);
  inclockdivider  = EEPROM.read(17);
  startstopmode   = EEPROM.read(18);
  sendclock       = EEPROM.read(19);
  osmoseenable    = EEPROM.read(20);

  /* READ TRACK PARATMETERS FROM EEPROM */
  for (i = 0; i <= maxTracks - 1; i++) {
    //laststep[i]     = EEPROM.read(16 + i);
    tracktype[i]      = EEPROM.read(32 + (0 * maxTracks) + i);
    ccnumber[i]       = EEPROM.read(32 + (1 * maxTracks) + i);
    clockdivider[i]   = EEPROM.read(32 + (2 * maxTracks) + i);
    outchannel[i]     = EEPROM.read(32 + (3 * maxTracks) + i);
    velocitysource[i]  = EEPROM.read(32 + (4 * maxTracks) + i);
    transposesource[i] = EEPROM.read(32 + (5 * maxTracks) + i);
    playonstart[i]    = EEPROM.read(32 + (6 * maxTracks) + i);
    /*The following sets basic settings*/
    velocity[i] = 96;
    transpose[i] = 0;
  }
}

void eepromfactoryreset() {
  clearscreen();
  lcd.setCursor(0, 0);
  lcd.print(F("Reset EEprom"));
  /* WRITE GENERIC PARATMETERS TO EEPROM */
  //EEPROM.update(4, 1); //outchannelall
  EEPROM.update(5, 1); //inchannel
  EEPROM.update(6, 0); //clockmode
  EEPROM.update(7, 60); //transposecenter
  EEPROM.update(8, 3); //transposemodeglobal
  EEPROM.update(9, 1); // noteoffmode
  //EEPROM.update(10, 30); //notelength
  EEPROM.update(11, 3); //notethrumode
  EEPROM.update(12, 1); //controlthrumode
  EEPROM.update(13, 0); //programthrumode
  EEPROM.update(14, 1); //bendthrumode
  EEPROM.update(15, 0); //otherthrumode
  EEPROM.update(16, 1); //velocitymodeglobal

  /* WRITE TRACK PARATMETERS TO EEPROM */
  for (i = 0; i < maxTracks; i++) {
    EEPROM.update(32 + (0 * maxTracks) + i, 0); //tracktype[i]
    EEPROM.update(32 + (1 * maxTracks) + i, 7); //ccnumber[i]
    EEPROM.update(32 + (2 * maxTracks) + i, i + 1); //clockdivider[i]
    EEPROM.update(32 + (3 * maxTracks) + i, i + 1); //outchannel[i]
    EEPROM.update(32 + (4 * maxTracks) + i, maxTracks + 2); //velocitysource[i]
    EEPROM.update(32 + (5 * maxTracks) + i, maxTracks + 1); //transposesource[i]
    EEPROM.update(32 + (6 * maxTracks) + i, 1); //playonstart[i]
    /*The following sets basic settings*/
    velocity[i] = 96;
    transpose[i] = 0;
  }
}

void loadfactorysequence() {
  lcd.setCursor(0, 0);
  lcd.print(F("RESET PATTERN"));
  lcd.setCursor(1, 1);
  lcd.print(F("               "));
  byte track;
  for (track = 0; track < maxTracks; track++) {
    for (i = 0; i < maxSteps; i++) {
      stepvalue[i][track] = 128;
    }
  }
  stepvalue[0][0] = 48;
  stepvalue[0][1] = 52;
  stepvalue[0][2] = 52;
  stepvalue[0][3] = 59;
  stepvalue[0][4] = 48 + 12;
  stepvalue[0][5] = 52 + 12;
  stepvalue[0][6] = 55 + 12;
  stepvalue[0][7] = 59 + 12;
  stepvalue[0][8] = 48 + 24;
  stepvalue[0][9] = 52 + 24;
  stepvalue[0][10] = 55 + 24;
  stepvalue[0][11] = 59 + 24;
  stepvalue[0][12] = 48 + 36;
  stepvalue[0][13] = 52 + 36;
  stepvalue[0][14] = 55 + 36;
  stepvalue[0][15] = 59 + 36;
  for (track = 0; track < maxTracks; track++) {
    laststep[track] = 2;
  }
}


void deletesequence() {
  lcd.setCursor(0, 0);
  lcd.print(F("DELETE PATTERN"));
  lcd.setCursor(1, 1);
  lcd.print(F("               "));
  delay(800);
  for (i = 0; i <= maxTracks - 1; i++) {
    for ( stepnumber[i] = 0; stepnumber[i] <= maxSteps - 1; stepnumber[i]++) {
      stepvalue[stepnumber[i]][i] = 128;
    }
  }
  for (i = 0; i <= maxTracks - 1; i++) {
    laststep[i] = 1;
  }
  clearscreen();
}
